@@ -2,7 +2,9 @@ package Catmandu::BagIt::Payload;
 use Moo;
 use IO::String;
 
-has 'name' => (is => 'ro');
+our $VERSION = '0.07';
+
+has 'filename' => (is => 'ro');
 has 'data' => (is => 'ro');
 has 'flag' => (is => 'rw', default => 0);
 
