@@ -120,6 +120,8 @@ use Catmandu::Util qw(:is);
 use Catmandu::BagIt;
 use Moo;
 
+our $VERSION = '0.07';
+
 with 'Catmandu::Importer';
 
 has bags              => (is => 'ro' , required => 1);
@@ -171,7 +173,7 @@ sub read_bag {
     }
 
     if ($self->include_payloads) {
-        $item->{payload_files}     = [ map { "data/" . $_->name } $bagit->list_files ];
+        $item->{payload_files}     = [ map { "data/" . $_->filename } $bagit->list_files ];
         $item->{non_payload_files} = [ $bagit->list_tagsum ];
     }
 
